#ifndef SCRIPTABLE_CONTROLLER_H
#define SCRIPTABLE_CONTROLLER_H

#include <lua.hpp>
#include <string>
#include <list>
#include <map>
#include <memory>
#include "scriptable_controller_api.h"
#include "core/CoreIncludes.h"
#include "worldentities/WorldEntity.h"
#include "worldentities/ControllableEntity.h"

struct lua_State;

namespace orxonox
{

class ScriptableController
{
public:
    struct NearObjectHandler
    {
        NearObjectHandler(WorldEntity *otherObject, double distance, std::function<void (int, int)> callback)
            : otherObject_(otherObject), distance_(distance), callback_(callback)
        {}

        WorldEntity *otherObject_;
        double distance_;
        std::function<void (int, int)> callback_;
        std::list<std::unique_ptr<NearObjectHandler> >::iterator otherHandler_;
    };

    int runScript(const std::string &file_path);

    void setPlayer(PlayerInfo *player);
    void registerWorldEntity(int id, WorldEntity *obj);
    void registerControllableEntity(int id, ControllableEntity *obj);

    WorldEntity *getWorldEntityByID(int id) const;
    ControllableEntity *getControllableEntityByID(int id) const;

    void addNearObjectHandler(int obj1, int obj2, double distance, std::function<void (int, int)> callback);

    void objectMoved(WorldEntity *obj);

private:
    std::list<std::unique_ptr<ScriptableControllerAPI> > apis_;
    PlayerInfo *player_;
    std::map<int, WorldEntity*> worldEntities_;
    std::map<int, ControllableEntity*> controllabelEntities_;

    std::map<WorldEntity*, std::list<std::unique_ptr<NearObjectHandler> > > nearObjectHandlers_;

    void printLuaError(lua_State *lua);
};

}

#endif // SCRIPTABLE_CONTROLLER_H
